from major_system import major_system as ms
import pytest

@pytest.fixture
def dictfile():
  return ['abc', 'dog', 'Cat', 'tag', 'toggle', 'at', 'yell', 'well',
      'what', 'tall', 'deal', 'gill', 'gal', 'ick']

def assert_equal_except_order(left, right):
  assert len(left) == len(right)
  for i in left:
    assert i in right

def test_partitions():
  assert ms.partitions("a") == [["a"]]
  assert ms.partitions("ab") == [["ab"], ["a", "b"]]
  assert ms.partitions("abc") == [["abc"], ["a", "bc"], ["ab", "c"], ["a", "b", "c"]]
  assert ms.partitions("abcd") == [["abcd"], ["a", "bcd"], ["ab", "cd"], ["abc", "d"], ["a", "b", "cd"], ["a", "bc", "d"], ["ab", "c", "d"], ["a", "b", "c", "d"]]
  assert ms.partitions("a", 0) == []
  assert ms.partitions("ab", 1) == [["ab"]]
  assert ms.partitions("abc", 0) == []
  assert ms.partitions("abc", 1) == [["abc"]]
  assert ms.partitions("abc", 2) == [["abc"], ["a", "bc"], ["ab", "c"]]
  assert ms.partitions("abc", 3) == [["abc"], ["a", "bc"], ["ab", "c"], ["a", "b", "c"]]
  assert ms.partitions("abc", 4) == [["abc"], ["a", "bc"], ["ab", "c"], ["a", "b", "c"]]
  assert ms.partitions("abc", 5) == [["abc"], ["a", "bc"], ["ab", "c"], ["a", "b", "c"]]
  assert ms.partitions("abc", 6) == [["abc"], ["a", "bc"], ["ab", "c"], ["a", "b", "c"]]
  assert ms.partitions("abc", 7) == [["abc"], ["a", "bc"], ["ab", "c"], ["a", "b", "c"]]
  assert ms.partitions("abc", None) == [["abc"], ["a", "bc"], ["ab", "c"], ["a", "b", "c"]]
  assert ms.partitions("abcd") == [["abcd"], ["a", "bcd"], ["ab", "cd"], ["abc", "d"], ["a", "b", "cd"], ["a", "bc", "d"], ["ab", "c", "d"], ["a", "b", "c", "d"]]


def test_major_words(dictfile):
  assert [x for x in ms.major_words(dictfile, 17)] == ["dog", 'tag']
  assert [x for x in ms.major_words(dictfile, 175)] == ["toggle"]
  assert [x for x in ms.major_words(dictfile, 8)] == []
  assert [x for x in ms.major_words(dictfile, 5)] == ['yell', 'well']


def test_phrases_from_partition(dictfile):
  partition = [175]
  phrases = [x for x in ms.phrases_from_partition(dictfile, partition)]
  assert phrases == ['toggle']
  partition = [17, 5]
  phrases = [x for x in ms.phrases_from_partition(dictfile, partition)]
  assert phrases == ['dog yell', 'dog well', 'tag yell', 'tag well']
  partition = [1, 75]
  phrases = [x for x in ms.phrases_from_partition(dictfile, partition)]
  assert phrases == ['at gill', 'at gal', 'what gill', 'what gal']
  partition = [1, 7, 5]
  phrases = [x for x in ms.phrases_from_partition(dictfile, partition)]
  assert phrases == ['at ick yell', 'at ick well', 'what ick yell',
      'what ick well']
  partition = [15]
  phrases = [x for x in ms.phrases_from_partition(dictfile, partition)]
  assert phrases == ['tall', 'deal']
  partition = [7]
  phrases = [x for x in ms.phrases_from_partition(dictfile, partition)]
  assert phrases == ['ick']
  partition = [1, 5]
  phrases = [x for x in ms.phrases_from_partition(dictfile, partition)]
  assert phrases == ['at yell', 'at well', 'what yell', 'what well']
  partition = [1]
  phrases = [x for x in ms.phrases_from_partition(dictfile, partition)]
  assert phrases == ['at', 'what']
  partition = [5]
  phrases = [x for x in ms.phrases_from_partition(dictfile, partition)]
  assert phrases == ['yell', 'well']


def test_phrases(dictfile):
  phrases = [x for x in ms.phrases(dictfile, 15)]
  assert phrases == [
    'tall', 'deal', 'at yell', 'at well', 'what yell', 'what well'
  ]
  phrases = [x for x in ms.phrases(dictfile, 175)]
  assert set(phrases) == set([
    'toggle',
    'dog yell', 'dog well', 'tag yell', 'tag well',
    'at gill', 'at gal', 'what gill', 'what gal',
    'at ick yell', 'at ick well', 'what ick yell',
    'what ick well',
  ])
  phrases = [x for x in ms.phrases(dictfile, 15, 1)]
  assert phrases == [
    'tall', 'deal',
  ]
  phrases = [x for x in ms.phrases(dictfile, 175, 3)]
  assert set(phrases) == set([
    'toggle',
    'dog yell', 'dog well', 'tag yell', 'tag well',
    'at gill', 'at gal', 'what gill', 'what gal',
    'at ick yell', 'at ick well', 'what ick yell',
    'what ick well',
  ])
  phrases = [x for x in ms.phrases(dictfile, 175, 2)]
  assert set(phrases) == set([
    'toggle',
    'dog yell', 'dog well', 'tag yell', 'tag well',
    'at gill', 'at gal', 'what gill', 'what gal',
  ])
  phrases = [x for x in ms.phrases(dictfile, 175, 1)]
  assert set(phrases) == set([
    'toggle',
  ])
  phrases = [x for x in ms.phrases(dictfile, 175, 0)]
  assert set(phrases) == set([
  ])



#!/usr/bin/env python3

import sys
import re
import itertools
import collections

__doc__ = """
Usage: {} [options] <number>...

<number>...                Numbers to turn into words
                           If -, take <number>... from STDIN.

Options:
  -d, --dict=<filename>    Dictionary file to use
                           [DEFAULT: /usr/share/dict/words]
  -m, --max-words=<number> Maximum number of words to split into
                           [DEFAULT: 3]
  -v, --version            Show version
""".format(sys.argv[0])

from docopt import docopt


def numeric_partitions(number, max_partitions):
  for partition in partitions(str(number), max_partitions):
    yield [int(x) for x in partition]


def regex_component(i):
  if i == 0:
    return r'[sSzZ]+'
  if i == 1:
    return r'[tTdD]+'
  if i == 2:
    return r'[nN]+'
  if i == 3:
    return r'[mM]+'
  if i == 4:
    return r'[rR]+'
  if i == 5:
    return r'[lL]+'
  if i == 6:
    return r'(j|J|sh|SH|ch|CH)'
  if i == 7:
    return r'[kKgGcC]+'
  if i == 8:
    return r'[fFvV]+'
  if i == 9:
    return r'[PpBb]+'
  if i == None:
    return r'[AaEeIiOoUuWwHhYy]*'
  raise ValueError("Expected a single digit or None")


def major_words(dictfile, number):
  # If it's something that can only be iterated through once (like a file
  # pointer), reset to the beginning again
  if isinstance(dictfile, collections.Iterator):
    dictfile.seek(0)
  regex = r'^' + regex_component(None)
  for character in str(number):
    regex += regex_component(int(character)) + regex_component(None)
  regex += r'$'
  for line in dictfile:
    if re.match(regex, line):
      yield line.strip()


def phrases_from_partition(dictfile, partition):
  word_collection = []
  for part in partition:
    words = major_words(dictfile, part)
    word_collection.append(words)
  for tup in itertools.product(*word_collection):
    yield " ".join(tup)


def phrases(dictfile, number, max_words=None):
  for numeric_partition in numeric_partitions(number, max_words):
    print(numeric_partition)
    for phrase in phrases_from_partition(dictfile, numeric_partition):
      yield phrase


def ordered_tuples(tuple_length, num_elts):
  if tuple_length == 1:
    for i in range(1, num_elts):
      yield (i,)
  else:
    for tup in ordered_tuples(tuple_length - 1, num_elts):
      for i in range(tup[-1] + 1, num_elts):
        yield tup + (i,)


def partitions(arr, max_partitions=None):
  has_max = False
  try:
    has_max = True
    max_partitions = int(max_partitions)
  except (TypeError, ValueError) as e:
    has_max = False

  if has_max and max_partitions < 1:
    return []
  num_elts = len(arr)
  # length 1
  to_return = [[arr]]
  # higher lengths
  for partition_length in range(2, num_elts + 1):
    if has_max and partition_length > max_partitions:
      continue
    tuple_length = partition_length - 1
    for tup in ordered_tuples(tuple_length, num_elts):
      partition = []
      partition.append(arr[0:tup[0]])
      for i in range(0, tuple_length - 1):
        partition.append(arr[tup[i]:tup[i + 1]])
      partition.append(arr[tup[-1]:num_elts])
      to_return.append(partition)
  return to_return


def main():
  args = docopt(__doc__, version='1.1.1')
  with open(args['--dict'], 'r') as dictfile:
    for number in args['<number>']:
      print("{}:".format(number))
      for phrase in phrases(dictfile, int(number), args['--max-words']):
        print("  " + phrase.strip())


if __name__ == "__main__":
  main()



from setuptools import setup

setup(
  name = "major_system",
  version = "1.1.1",
  author = "John Baber-Lucero",
  author_email = "pypi@frundle.com",
  description = ("Python libraries and scripts for converting numbers using the Mnemonic Major System"),
  license = "GPLv3",
  url = "https://github.com/jbaber/major_system",
  packages = ['major_system'],
  install_requires = ['docopt'],
  tests_require=['pytest'],
  entry_points = {
    'console_scripts': ['major_words=major_system.major_system:main'],
  }
)
